/* read in data for generating units */

#include <stdio.h>
#include <math.h>
#include <string.h>

/* define some constants */

#define MAXPERIODS    168
#define MAXSTATES     256
#define MAXNUMUNITS   50
#define MAXNUMCOMBOS  8
#define INFINITY      1.0E10
#define YES           1
#define NO            0
#define ONLINE        1
#define OFFLINE       0
#define PREV          0
#define PRES          1

/* A generating unit is uniquely defined by the following structure.  
   Elements of the structure are:

      1. minimum output in (MW) (MinOutput),
      2. maximum output in (MW) (MaxOutput),
      3. incremental heat rate in (Btu/mWh) (IncHeatRate),
      4. no-load cost in (Btu/h) (NoLoadCost),
      5. full-load average cost in ($/mWh) (FullLoadAveCost),
      6. fuel cost in ($/MBtu) FuelCost,
      7. minimum up time in (h) (MinUpTime),
      8. minimum down time in (h) (MinDownTime),
      9. hot start-up costs in ($) (HotStartCost),
     10. cold start-up-costs in (MBtu) (ColdStartCost),
     11. fixed cold start-up-costs in ($) (ColdStartFixedCost),
     12. cold start in (h) (ColdStart),
     13. thermal time constant (ThermTimeConst),
     14. cost of maintaining unit at operating temperature 
         in (Btu/h) (MaintainTempCost),
     15. indicator of whether unit is online or offline (OnOffLine[][]),
     16. time on or off line in (h) (TimeOnOffLine[][]),
     17. indicator of whether unit is must-run (MustRun[][]), 
     18. indicator of whether unit is available (Available[][]), and
     19. unit production in (MW) (UnitProduction[]).

   These unit characteristics are from table 5.4 in "Power Generation, 
   Operation and Control", and they will be modified to suit the 
   requirements of Prof. Y. Wallach.

   Some hysterias on generating unit information must be maintained, but 
   only for the previous state.  Consequently, OnOffLine[][], 
   TimeOnOffLine[][], MustRun[][], and Available[][] are for only 
   kept for the previous state and modified during the present state. */

struct Unit {float MinOutput;
             float MaxOutput;
             float IncHeatRate;
             float NoLoadCost;
             float FullLoadAveCost;
             float FuelCost;
             int   MinUpTime;
             int   MinDownTime;
             float HotStartCost;
             float ColdStartCost;
             float ColdStartFixedCost;
             int   ColdStart;
             float ThermTimeConst;
             float MaintainTempCost;
             int   OnOffLine[2][MAXSTATES];
             int   TimeOnOffLine[2][MAXSTATES];
             int   MustRun[2][MAXSTATES];
             int   Available[2][MAXSTATES];
             float UnitProduction[MAXSTATES]};


/* Declare an array of Units - units are numbered starting at 1 */

struct Unit Units[MAXNUMUNITS+1];


/* Declare an array of possible generating units */

int PossUnits[MAXNUMUNITS+1];

int NumUnits;     /* number of generating units to be examined */
int NumStates;    /* number of states to be examined per period */
int NumPeriods;   /* number of periods to be examined */
int NumCombos;    /* number of generating unit combinations to be 
                     examined.  NumStates should equal 2^NumCombos */


FILE *OutputFile; /* pointer to the output file stream */


/* Declare an array of load patterns for each of NumPeriods */

float LoadPattern[MAXPERIODS+1];


/* define user-supplied functions */
/* function names are designed to be self-explanatory */

void  InitStaticData();

/* operations in the main routine are abstracted by extensive use of
   functions. */

/* the main program is called with no arguments */

main()
 {
  int IPeriod;                  /* variable that represents the period 
                                   under examination */

  InitStaticData();             /* get all input data from a file */

 }


/* This function reads the file ucdpin.dat to input constants into this 
   program */

/* input file format:

NumPeriods
LoadPattern[0] LoadPattern[1] ... LoadPattern[NumPeriods]

NumStates
NumCombos
NumLTLoad

NumUnits
Units[1].MinOutput Units[1].MaxOutput ... 
   Units[1].OnOffLine[PRES][1] Units[1].TimeOnOffLine[PRES][1] 
Units[2].MinOutput Units[2].MaxOutput ... 
   Units[2].OnOffLine[PRES][1] Units[2].TimeOnOffLine[PRES][1] 
.
.
.
Units[NumUnits].MinOutput Units[NumUnits].MaxOutput ... 
   Units[NumUnits].OnOffLine[PRES][1] Units[NumUnits].TimeOnOffLine[PRES][1] 

  */

void InitStaticData()
 {
  int   IPeriod;  /* index variable for period */
  int   IState;   /* index variable for state */
  int   IUnit;    /* index variable for unit */

  float MinOutput;
  float MaxOutput;
  float IncHeatRate;
  float NoLoadCost;
  float FullLoadAveCost;
  float FuelCost;
  int   MinUpTime;
  int   MinDownTime;
  float HotStartCost;
  float ColdStartCost;
  float ColdStartFixedCost;
  int   ColdStart;
  float ThermTimeConst;
  float MaintainTempCost;
  int   OnOffLine;
  int   TimeOnOffLine;

  FILE *InputFile;   /* pointer to the input file stream */

/* open the output data file (global variable) */

  OutputFile = fopen("ucdpout.dat", "w");

/* open the input data file */

  InputFile = fopen("ucdpin.dat", "r");

/* input the number of periods to be examined */

  fscanf(InputFile, "%d", &NumPeriods);

/* input the load pattern, one value for each period */

  for(IPeriod = 1; IPeriod <= NumPeriods; IPeriod++)
   {
    fscanf(InputFile, "%f", (LoadPattern + IPeriod));
   }

/* input the number of states to be examined */

  fscanf(InputFile, "%d", &NumStates);

/* input the number of combinations to be examined */

  fscanf(InputFile, "%d", &NumCombos);

/* input the number of generating units less than the number required 
   for load plus spinning reserve to be examined */

  fscanf(InputFile, "%d", &NumLTLoad);

/* input the number of generating units to be examined */

  fscanf(InputFile, "%d", &NumUnits);

/* input the generating unit descriptions */

  for(IUnit = 1; IUnit <= NumUnits; IUnit++)
   {
    fscanf(InputFile, "%f %f %f %f %f %f %d %d %f %f %f %d %f %f %d %d",
                         &MinOutput,
                         &MaxOutput,
                         &IncHeatRate,
                         &NoLoadCost,
                         &FullLoadAveCost,
                         &FuelCost,
                         &MinUpTime,
                         &MinDownTime,
                         &HotStartCost,
                         &ColdStartCost,
                         &ColdStartFixedCost,
                         &ColdStart,
                         &ThermTimeConst,
                         &MaintainTempCost,
                         &OnOffLine,
                         &TimeOnOffLine);

    Units[IUnit].MinOutput               = MinOutput;
    Units[IUnit].MaxOutput               = MaxOutput;
    Units[IUnit].IncHeatRate             = IncHeatRate;
    Units[IUnit].NoLoadCost              = NoLoadCost;
    Units[IUnit].FullLoadAveCost         = FullLoadAveCost;
    Units[IUnit].FuelCost                = FuelCost;
    Units[IUnit].MinUpTime               = MinUpTime;
    Units[IUnit].MinDownTime             = MinDownTime;
    Units[IUnit].HotStartCost            = HotStartCost;
    Units[IUnit].ColdStartCost           = ColdStartCost;
    Units[IUnit].ColdStartFixedCost      = ColdStartFixedCost;
    Units[IUnit].ColdStart               = ColdStart;
    Units[IUnit].ThermTimeConst          = ThermTimeConst;
    Units[IUnit].MaintainTempCost        = MaintainTempCost;
    Units[IUnit].OnOffLine[PRES][0]      = OnOffLine;
    Units[IUnit].TimeOnOffLine[PRES][0]  = TimeOnOffLine;

/* copy the on-off condition to the entire array for both the first 
   period and the initial period */

    Units[IUnit].OnOffLine[PREV][0]     = 
           Units[IUnit].OnOffLine[PRES][0];

    Units[IUnit].TimeOnOffLine[PREV][0] = 
           Units[IUnit].TimeOnOffLine[PRES][0];

    /* BinaryRep(0,0);

    for (IState = 1; IState < NumStates; IState++)
     {
      Units[IUnit].OnOffLine[PRES][IState]       =
             Units[IUnit].OnOffLine[PRES][0];

      Units[IUnit].OnOffLine[PREV][IState]       =
             Units[IUnit].OnOffLine[PRES][0];

      Units[IUnit].TimeOnOffLine[PRES][IState]   =
             Units[IUnit].TimeOnOffLine[PRES][0];

      Units[IUnit].TimeOnOffLine[PREV][IState]   =
             Units[IUnit].TimeOnOffLine[PRES][0];

      BinaryRep(0,IState); 
     }
    */
   }

  fclose(InputFile);
 }

